﻿cd "Demo 1 - Deploying Content Types and Site Columns to a site collection"
[xml] $config = Get-Content -LiteralPath 'Demo1Config.xml'

##FUNCTIONS

function createContentType($ctypeEntry)
{

        $flag = $false
        foreach ($scCT in $scCTypes)
        {
            if ($ctypeEntry.Id -eq $scCT.Id)
            {
                $flag = $true
                break
            }          
        }
        if ($flag)
        {
            write-host "Content type exists" -ForegroundColor green
        }
        else 
        {
            write-host "Content type does not exist, creating content type"
            
            $newCt = new-object Microsoft.SharePoint.Client.ContentTypeCreationInformation
            $newCt.Name = $ctypeEntry.Name
            $newCt.Group = $ctypeEntry.Group
            $newCt.Id = $ctypeEntry.Id
            $newCType = $scCTypes.Add($newCt)
            $scContext.Load($newCType)
            $scContext.ExecuteQuery()

            ##Add the required site columns
            foreach ($fieldToAdd in $ctypeEntry.fields.field)
            {
                $siteColumnRef = $siteCols.GetByInternalNameOrTitle($fieldToAdd.Name)

                write-host "     Adding" $fieldToAdd.Name "to content type"

                try
                {
                    $scContext.Load($siteCols)
                    $scContext.load($newCType.FieldLinks)
                    $scContext.load($newCType.Fields)

                    $fieldRefLink = new-Object Microsoft.SharePoint.Client.FieldLinkCreationInformation
                    $fieldRefLink.Field = $siteColumnRef
                    $fieldRefLink.Field.Required = $fld.Required

                    ##Using an outputsink variable to surpress some usless info.
                    $outputSink = $newCType.FieldLinks.Add($fieldRefLink)
                    $newCType.Update($true)
                    $scContext.ExecuteQuery()
                }
                catch
                {
                    write-host "***ERROR*** Unable to add Column" $fld.Name "as it could not be found" -ForegroundColor Red
                }

            }
        }
}

##END OF FUNCTIONS


#########################
#### START OF SCRIPT ####
#########################

##Add the types for our Client DLLS
Add-Type -path .\Microsoft.SharePoint.Client.dll
Add-type -path .\Microsoft.SharePoint.Client.Runtime.dll
Add-type -path .\Microsoft.SharePoint.Client.Taxonomy.dll

##Setup our security context to connect to SPO
$baseURL = $config.Demo1.Tenant.BaseUrl
$adminURL = $config.Demo1.Tenant.AdminUrl
$AdminUser = $config.Demo1.Tenant.AdminUser

$siteCollectionURL = $baseURL + $config.Demo1.SiteCollection.URL

if (!$Credentials)
{
    $Credentials = Get-Credential -UserName $AdminUser -Message "Enter the password for $AdminUser"
}

##First we connect using Connect-SPService to validate our credentials
##Connect-SPOService -Url $adminURL -Credential $credentials

##Then we'll establish a ClientContext for CSOM.
$scContext = New-Object Microsoft.SharePoint.Client.ClientContext($siteCollectionURL)
$SPOcredentials = New-Object Microsoft.SharePoint.Client.SharePointOnlineCredentials($Credentials.UserName, $Credentials.Password)
$scContext.Credentials = $SPOcredentials

##Now we'll validate whether or not our Site Columns are present.
##If they're not, we'll add them.

$web = $scContext.Web
$siteCols = $web.Fields
$scCTypes = $web.ContentTypes 

$scContext.load($web)
$scContext.load($scCTypes)
$scContext.load($siteCols)
$scContext.ExecuteQuery()

write-host("########### Creating Site Columns ###############") -ForegroundColor Green


foreach ($siteColumn in $config.Demo1.SiteCollection.SiteColumns.SiteColumn)
{
    write-host "Validating" $siteColumn.DisplayName

    $flagColumnExists = $false

    foreach ($siteCol in $siteCols)
    {
        if ($siteCol.InternalName -eq $siteColumn.Name)
        {
            $flagColumnExists = $true
            break
        }
    }

    if ($flagColumnExists)
    {
        write-host "The site column" $siteColumn.displayname "already exists" -ForegroundColor green
    }
    else
    {
        write-host "Creating site column:-" $siteColumn.DisplayName -ForegroundColor Green
        
        $fieldAsXML = new-object System.Xml.XmlDocument

        $fieldElement = $fieldAsXML.CreateElement("Field")
        $fieldElement.SetAttribute("DisplayName",$siteColumn.DisplayName)
        $fieldElement.SetAttribute("Name", $siteColumn.Name)
        $FieldElement.SetAttribute("Group", "_EVO 2015")
        $fieldElement.SetAttribute("Type",$siteColumn.Type)
        $fieldElement.SetAttribute("ID",$siteColumn.Id)
        
        
        $flagFieldComplete = $false

        switch ($siteColumn.Type)
        {
            "Choice"
            {
                $choicesRoot = $fieldAsXML.CreateElement("CHOICES")

                foreach ($choiceOption in $siteColumn.Choices.Choice)
                {
                    $choiceOptionElement = $fieldAsXML.CreateElement("CHOICE")
                    $choiceOptionValue = $fieldAsXML.CreateTextNode($choiceOption.value)              
                    $choiceOptionElement.AppendChild($choiceOptionValue)
                    $choicesRoot.AppendChild($choiceOptionElement)
                }

                $fieldElement.AppendChild($choicesRoot)
                $flagFieldComplete = $true
            }

            "TaxonomyFieldTypeMulti"
            {
                ##Do nothing extra, until the initial column is created.
            }

            default
            {                
                $flagFieldComplete = $true
            }
        }
        
        $fieldAsXml.AppendChild($fieldElement)

        ##Finished building the XML, so now add the column with options
        ##Note the AddFieldInternalNameHint is important to ensure we get our nice Internal names!
        $fieldOption = [Microsoft.SharePoint.Client.AddFieldOptions]::AddFieldInternalNameHint

        $newSiteColumn = $siteCols.AddFieldAsXML($fieldAsXML.InnerXML, $false, $fieldOption) 
        $scContext.load($newSiteColumn)
        
        try
        {
            $scContext.ExecuteQuery()

            if (!$flagFieldComplete)
            {

                switch ($siteColumn.Type)
                {
                    #Additional config is required after the field is created for some column types
                    "TaxonomyFieldTypeMulti"
                    {
                        $MMS = [Microsoft.SharePoint.Client.Taxonomy.TaxonomySession]::GetTaxonomySession($scContext)
                        $scContext.Load($MMS)
                        $scContext.Load($MMS.termstores)
                        $scContext.ExecuteQuery()

                        $scContext.load($MMS.termstores[0])
                        $scContext.load($MMS.termstores[0].Groups)
                        $scContext.ExecuteQuery()
                        $termStoreID = $MMS.TermStores[0].id

                        $group = $MMS.termstores[0].groups.GetByName($config.Demo1.ManagedMetadata.FormClassification.Group)
                        $scContext.load($group)
                        $scContext.ExecuteQuery()
                        $termset = $group.TermSets.GetByName($config.Demo1.ManagedMetadata.FormClassification.TermSet)
                        $scContext.load($termSet)
                        $scContext.ExecuteQuery()

                        ##Invoke the CastTo as a Generic
                        $taxField = [Microsoft.SharePoint.Client.ClientContext].GetMethod("CastTo").MakeGenericMethod([Microsoft.SharePoint.Client.Taxonomy.TaxonomyField]).Invoke($scContext, $newSiteColumn)

                        $taxField.SspId = $termStoreID
                        $taxField.TermSetId = $termSet.ID
                        $taxField.TargetTemplate = ""
                        $taxField.AnchorId = [System.Guid]::Empty
                        $taxField.Update()
                        $scContext.ExecuteQuery()

                    }

                    default
                    {
                    }
                }
            }
        }
        catch
        {
            ##Error creating the new column.. report this back.
            write-host "Error creating site column" $siteColumn.DisplayName
            return $false
        }
    }

}


##Now create the Content Types.
##Base types first, then children. (Note: This only allows for one layer of inheritence..)

write-host("########### Creating Content Types ###############") -ForegroundColor DarkYellow

foreach ($basecType in $config.demo1.SiteCollection.ContentTypes.BaseContentType)
{
    write-host "Validating" $basecType.name "content type"
    createContentType($basecType)
}

##And now the child content types (Note: Inheritence is denoted by the Content Type IDs)

foreach ($childcType in $config.demo1.SiteCollection.ContentTypes.ChildContentType)
{
    write-host "Validating" $childcType.name "content type"
    createContentType($childcType)
    
}

  

##Now return to the parent directory for the next demo
cd ..
