﻿cd "Demo 2 - Deploying the form manager list, items and display template"
[xml] $config = Get-Content -LiteralPath 'Demo2Config.xml'

##FUNCTIONS


##END OF FUNCTIONS


#########################
#### START OF SCRIPT ####
#########################

##Add the types for our Client DLLS
Add-Type -path .\Microsoft.SharePoint.Client.dll
Add-type -path .\Microsoft.SharePoint.Client.Runtime.dll
Add-type -path .\Microsoft.SharePoint.Client.Taxonomy.dll

##Setup our security context to connect to SPO
$baseURL = $config.Demo2.Tenant.BaseUrl
$adminURL = $config.Demo2.Tenant.AdminUrl
$AdminUser = $config.Demo2.Tenant.AdminUser

$siteCollectionURL = $baseURL + $config.Demo2.SiteCollection.URL

if (!$Credentials)
{
    $Credentials = Get-Credential -UserName $AdminUser -Message "Enter the password for $AdminUser"
}

##First we connect using Connect-SPService to validate our credentials
##Connect-SPOService -Url $adminURL -Credential $credentials

##Then we'll establish a ClientContext for CSOM.
$scContext = New-Object Microsoft.SharePoint.Client.ClientContext($siteCollectionURL)
$SPOcredentials = New-Object Microsoft.SharePoint.Client.SharePointOnlineCredentials($Credentials.UserName, $Credentials.Password)
$scContext.Credentials = $SPOcredentials

$web = $scContext.Web
$scLists = $web.lists

$scContext.load($web)
$scContext.load($scLists)
$scContext.ExecuteQuery()

$targetListExists = $false

write-host("########### Checking Form Manager List ###############") -ForegroundColor DarkYellow

foreach ($chkList in $scLists)
{
    if ($chkList.title -eq "Form Manager")
    {
        $targetListExists = $true
        break
    }
}

if ($targetListExists)
{
    write-host "########### Form Manager list already exists ###############" -ForegroundColor red
}
else
{
    write-host "########### Creating Form Manager List ###############" -ForegroundColor Green


    [Microsoft.SharePoint.Client.ListCreationInformation] $newListInfo = new-object Microsoft.SharePoint.Client.ListCreationInformation
    $newListInfo.Title = $config.demo2.sitecollection.FormManager.name
    $newListInfo.Description = $config.demo2.sitecollection.FormManager.description
    $newListInfo.TemplateType = [int][Microsoft.SharePoint.Client.ListTemplateType]::GenericList
    $createdList = $scLists.Add($newListInfo)
    $createdList.EnableAttachments = $false
    $createdList.EnableFolderCreation = $false
    $createdList.ContentTypesEnabled = $true
    $createdList.Update()
    $scContext.ExecuteQuery()


    write-host "########### Adding content types ###############" -ForegroundColor Green
    ##Now add the content types
    $scCtypes = $web.ContentTypes
    $listCtypes = $createdList.ContentTypes

    $scContext.Load($createdList)
    $scContext.Load($scCtypes)
    $scContext.Load($listCtypes)

    foreach ($cType in $config.demo2.sitecollection.FormManager.ContentTypes.ContentType)
    {
        $siteContentType = $scCtypes.GetById($cType.ID)
        $scContext.load($siteContentType)
        $ctReturn = $listCtypes.AddExistingContentType($siteContentType)

    }

    $scContext.ExecuteQuery()

    ##Now remove the Item content type
    foreach ($listCType in $listCtypes)
    {
        if ($listCType.name -eq "Item")
        {
            $listCType.DeleteObject()
            $scContext.ExecuteQuery()
            break
        }
    }

    write-host "########### Building views ###############" -ForegroundColor Green
    ##And now add any required views
    foreach ($viewEntry in $config.demo2.SiteCollection.FormManager.View)
    {
        ##Build a new view
        [Microsoft.SharePoint.Client.ViewCreationInformation] $newListView = New-Object Microsoft.SharePoint.Client.ViewCreationInformation
        $newListView.ViewTypeKind = [Microsoft.SharePoint.Client.ViewType]::None
        $newListView.RowLimit = $viewEntry.RowLimit
        $newListView.Paged = [Bool]$viewEntry.Paged
        $newListView.Title = $viewEntry.Name
        $newListView.SetAsDefaultView = [Bool]$viewEntry.SetAsDefaultView

        $fldCol = new-object System.Collections.Specialized.StringCollection
        foreach ($fld in $viewEntry.ViewFields.Split(","))
        {
            $outputsink = $fldCol.Add($fld)
        }

        $newListView.ViewFields = $fldCol
        
        $createdView = $createdList.Views.Add($newListView)
        if ($viewEntry.JSLink)
        {
            $createdView.JSLink = $viewEntry.JSLink
        }
        $createdView.Update()
        $scContext.load($createdList.Views)
        $scContext.load($createdView)
        $scContext.ExecuteQuery()
    }

    write-host "########### Uploading supporting files ###############" -ForegroundColor Green
    ##And now upload any supporting files
    foreach ($fileEntry in $config.demo2.SiteCollection.SupportingFiles.file)
    {
        $library = $scLists.GetByTitle($fileEntry.library)
        $rootFolder = $library.RootFolder
        $allCtypes = $library.ContentTypes

        $scContext.Load($library)
        $scContext.load($rootFolder)
        $scContext.load($allCtypes)
        $scContext.ExecuteQuery()
        $subFolder = $fileEntry.folder

        $docCtype = $allCtypes | ?{$_.name -eq "Document"}

        

        $filePath = (get-location).path + "\" + $fileEntry.filename
        $fileStream = [System.IO.File]::ReadAllBytes($filePath)

        [Microsoft.SharePoint.Client.FileCreationInformation] $fileCreationInfo = New-Object Microsoft.SharePoint.Client.FileCreationInformation;
        $fileCreationInfo.Overwrite = [Bool] $fileEntry.overwrite
        $fileCreationInfo.Content = $fileStream
        $fileCreationInfo.Url = $rootFolder.ServerRelativeUrl + "/" + $subFolder + "/" + $fileEntry.filename

        $folder = $rootFolder

        if (!$subFolder -eq "/")
        {
            $folder = $rootFolder.Folders.GetByUrl($subFolder)
        }
        
        [Microsoft.SharePoint.Client.File] $uploadFile = $folder.Files.Add($fileCreationInfo)
        $scContext.Load($uploadFile)

        $listItem = $uploadFile.ListItemAllFields

        $listItem["ContentTypeId"] = $docCtype.Id.StringValue
        $listItem["Title"] = $fileEntry.filename.Substring(0,$fileEntry.filename.LastIndexOf("."))
        ##$listItem[""]
        $listItem.Update()
        $scContext.ExecuteQuery()

    }

    write-host "########### Creating sample items ###############" -ForegroundColor Green
    ##And create a couple of sample items
    $formManagerList = $web.lists.GetByTitle("Form Manager")
   
    foreach ($sampleItem in $config.Demo2.SiteCollection.FormManager.Items.Item)
    {
        [Microsoft.SharePoint.Client.ListItemCreationInformation] $newlistitemInfo = New-Object Microsoft.SharePoint.Client.ListItemCreationInformation
        $newListItem = $formManagerList.AddItem($newlistitemInfo)
        $newListItem["Title"] = $sampleItem.title
        $newListitem["EVOFormDescription"] = $sampleItem.description
        $newListItem["EVOFormURL"] = $sampleitem.formurl
        $newListItem["TargetBehaviour"] = $sampleitem.targetbehaviour

        $newlistitem.Update()

    }

    $scContext.ExecuteQuery()


}

##Now return to the parent directory for the next demo
cd ..