﻿##Export Classification Term Set
##This script must be run after the ConfigureISE script

##Thanks for Brendan Griffin - Microsoft for his blog post saving me a ton of time!

[xml] $config = Get-Content -LiteralPath 'TaxonomyConfig.xml'

#########################
#### START OF SCRIPT ####
#########################

##Add the types for our Client DLLS
Add-Type -path .\Microsoft.SharePoint.Client.dll
Add-type -path .\Microsoft.SharePoint.Client.Runtime.dll
Add-type -path .\Microsoft.SharePoint.Client.Taxonomy.dll

##Setup our security context to connect to SPO
$baseURL = $config.TaxonomyConfig.Tenant.BaseUrl
$adminURL = $config.TaxonomyConfig.Tenant.AdminUrl
$AdminUser = $config.TaxonomyConfig.Tenant.AdminUser

$siteCollectionURL = $baseURL

if (!$Credentials)
{
    $Credentials = Get-Credential -UserName $AdminUser -Message "Enter the password for $AdminUser"
}

##First we connect using Connect-SPService to validate our credentials
Connect-SPOService -Url $adminURL -Credential $credentials

# connect/authenticate to SharePoint Online and get ClientContext object.. 
$mmsContext = New-Object Microsoft.SharePoint.Client.ClientContext($baseurl) 
$SPOcredentials = New-Object Microsoft.SharePoint.Client.SharePointOnlineCredentials($Credentials.UserName, $Credentials.Password) 
$mmsContext.Credentials = $SPOcredentials

#Bind to MMS
#NOTE: This next line will likely FAIL if you run it in PowerShell ISE...
$MMS = [Microsoft.SharePoint.Client.Taxonomy.TaxonomySession]::GetTaxonomySession($mmsContext)
$mmsContext.Load($MMS)
$mmsContext.ExecuteQuery()

#Retrieve Term Stores
$termStoreCollection = $MMS.TermStores
$mmsContext.Load($termStoreCollection)
$mmsContext.ExecuteQuery()

#Bind to Term Store
$termStore = $TermStoreCollection[0]
$mmsContext.Load($TermStore)
$mmsContext.ExecuteQuery()

#Retrieve Groups
$termGroups = $termStore.Groups
$mmsContext.load($termGroups)
$mmsContext.ExecuteQuery()

function enumerateTerm ($parentTerm, $parentTermNode, $indent)
{
    $childTerms = $parentTerm.Terms
    $mmsContext.Load($childTerms)
    $mmsContext.ExecuteQuery()

    foreach ($trm in $childTerms)
    {
        write-host $indent  $trm.TermsCount " - " $trm.Name  "  -  " $trm.id -ForegroundColor white
        $enumTermNode = $outputTerms.CreateElement("Term")
        $enumTermNode.SetAttribute("Name",$trm.Name)
        $enumTermNode.SetAttribute("TermCount",$trm.TermsCount)
        $enumTermNode.SetAttribute("ID",$trm.id)

        if ($trm.TermsCount -ge 1)
        {
            enumerateTerm $trm $enumTermNode ($indent + "   ")
        }

        $parentTermNode.appendChild($enumTermNode)
    }

}


$outputTerms = New-Object System.Xml.XmlDocument
$rootNode = $outputTerms.CreateElement("ManagedMetadata")



#Retrieve Term Sets in each group
foreach ($grp in $termGroups)
{
    if ($grp.name -eq $config.TaxonomyConfig.ManagedMetadata.FormClassification.Group)
    {
        $groupNode = $outputTerms.CreateElement("Group")
        $groupNode.SetAttribute("Name",$grp.name)

        
        $mmsContext.load($grp)
        $mmsContext.load($grp.TermSets)
        $mmsContext.ExecuteQuery()
        write-host "Group Name:" $grp.name -foregroundcolor Green
        write-host "Group ID  :" $grp.Id -ForegroundColor Green
        
        foreach ($termSet in $grp.TermSets)
        {
            write-host "Term Set Name:" $termSet.Name -ForegroundColor Yellow
            write-host "Term Set ID  :" $termSet.Id -ForegroundColor Yellow
            write-host "    Terms:" -ForegroundColor DarkCyan

            $termsetNode = $outputTerms.CreateElement("Termset")
            $termsetNode.SetAttribute("Name",$termSet.Name)
            $termsetNode.SetAttribute("ID",$termset.Id)

            $terms = $termSet.terms
            $mmsContext.Load($terms)
            $mmsContext.ExecuteQuery()

            foreach ($term in $terms)
            {
                write-host "  " $term.TermsCount " - " $term.Name "  -  " $term.id -ForegroundColor white
                $termNode = $outputTerms.CreateElement("Term")
                $termNode.SetAttribute("Name",$term.Name)
                $termNode.SetAttribute("TermCount",$term.TermsCount)
                $termNode.SetAttribute("ID",$term.id)

                
                if ($term.TermsCount -ge 1)
                {
                    enumerateTerm $term $termNode "     "
                }
                
                $termsetnode.AppendChild($termNode)
                
            }

            $groupNode.AppendChild($termsetNode)

        }

        $rootnode.AppendChild($groupNode)

    }
}

$outputterms.AppendChild($rootNode)


$StringWriter = New-Object System.IO.StringWriter 
$XmlWriter = New-Object System.XMl.XmlTextWriter $StringWriter 
$xmlWriter.Formatting = "indented" 
$outputTerms.WriteTo($XmlWriter) 
$XmlWriter.Flush() 
$StringWriter.Flush() 
Write-Output $StringWriter.ToString()

$outputTerms.Save("$pwd\termsetdata.xml")
