﻿##Export Classification Term Set
##NOTE: You must have been added to the Term Store admin group!

##Thanks for Brendan Griffin - Microsoft for his blog post saving me a ton of time!

[xml] $config = Get-Content -LiteralPath 'ImportTaxonomyConfig.xml'
[xml] $termSetData = Get-Content -LiteralPath 'termsetdata.xml'

#########################
#### START OF SCRIPT ####
#########################

##Add the types for our Client DLLS
Add-Type -path .\Microsoft.SharePoint.Client.dll
Add-type -path .\Microsoft.SharePoint.Client.Runtime.dll
Add-type -path .\Microsoft.SharePoint.Client.Taxonomy.dll

##Setup our security context to connect to SPO
$baseURL = $config.TaxonomyConfig.Tenant.BaseUrl
$adminURL = $config.TaxonomyConfig.Tenant.AdminUrl
$AdminUser = $config.TaxonomyConfig.Tenant.AdminUser

$siteCollectionURL = $baseURL

if (!$Credentials)
{
    $Credentials = Get-Credential -UserName $AdminUser -Message "Enter the password for $AdminUser"
}

##First we connect using Connect-SPService to validate our credentials
##Connect-SPOService -Url $adminURL -Credential $credentials

# connect/authenticate to SharePoint Online and get ClientContext object.. 
$mmsContext = New-Object Microsoft.SharePoint.Client.ClientContext($baseurl) 
$SPOcredentials = New-Object Microsoft.SharePoint.Client.SharePointOnlineCredentials($Credentials.UserName, $Credentials.Password) 
$mmsContext.Credentials = $SPOcredentials

#Bind to MMS
#NOTE: This next line will likely FAIL if you run it in PowerShell ISE...
$MMS = [Microsoft.SharePoint.Client.Taxonomy.TaxonomySession]::GetTaxonomySession($mmsContext)
$mmsContext.Load($MMS)
$mmsContext.ExecuteQuery()

#Retrieve Term Stores
$termStoreCollection = $MMS.TermStores
$mmsContext.Load($termStoreCollection)
$mmsContext.ExecuteQuery()

#Bind to Term Store
$termStore = $TermStoreCollection[0]
$mmsContext.Load($TermStore)
$mmsContext.ExecuteQuery()

#Retrieve Groups
$termGroups = $termStore.Groups
$mmsContext.load($termGroups)
$mmsContext.ExecuteQuery()

###############
## FUNCTIONS ##
###############

function enumerateTerm($parentTermSet, $parentTerm, $indent)
{
    $subindent = $indent + "  "
    foreach ($childterm in $parentTerm.term)
    {

        $termID = $childterm.id
        if (!$termID)
        {
            $termID = [System.Guid]::NewGuid().ToString()
        }

        write-host $indent $childterm.name "  -  " $termID
        $termAdd = $parentTermSet.CreateTerm($childterm.name,1033,$termID)
        $mmsContext.Load($termAdd)
        $mmsContext.ExecuteQuery() 

        if ($childterm.termcount -gt 0)
        {
            enumerateTerm $termAdd $childterm $subindent
        }
    }
}


######################
## End of FUNCTIONS ##
######################

#Process groups
foreach ($group in $termSetData.ManagedMetadata.Group)
{
    $groupExists = $false

    ##First we check if the group already exists
    foreach ($grp in $termGroups)
    {
        if ($grp.name -eq $config.TaxonomyConfig.ManagedMetadata.FormClassification.Group)
        {
            $groupExists = $true
        }
    }

    if (!$groupExists)
    {
        write-host $config.TaxonomyConfig.ManagedMetadata.FormClassification.Group "Group does not exist. - CREATING" -ForegroundColor green

        $newGroup = $termStore.CreateGroup($config.TaxonomyConfig.ManagedMetadata.FormClassification.Group,[System.Guid]::NewGuid().toString())

        $mmsContext.Load($newGroup)
        $mmsContext.ExecuteQuery()

        ##Now create the term sets and terms
        foreach ($termset in $termSetData.ManagedMetadata.Group.Termset)
        {
            Write-Host "Creating term set " $termSet.name
            $createdTermSet = $newGroup.CreateTermSet($termset.Name,$termset.ID,1033)
            $mmsContext.Load($createdTermSet)
            $mmsContext.ExecuteQuery()

            write-host $termset.Name
            foreach ($term in $termset.term)
            {
                $termID = $term.id
                if (!$termID)
                {
                    $termID = [System.Guid]::NewGuid().ToString()
                }
                
                $childtermAdd = $createdTermSet.CreateTerm($term.name,1033,$termID)
                $mmsContext.Load($childtermAdd)
                $mmsContext.ExecuteQuery() 
                enumerateTerm $childtermAdd $term "   "
            }
        }
    }
    else
    {
        Write-Host $group "already exists - SKIPPING"
    }

}